#-----------------------------------------------------------------------------
# Z-flatten.py
# v3
# 7/22/2003 by Ockham
# revised 4/15/04
#-----------------------------------------------------------------------------
""" Stewer's idea:

Create two Ball props, Ball_A and Ball_B, and a figure. Then create the following hierarchy:

-Universe
...-Ball_A
......-Ball_B
.........+Figure_1

    Then animate Figure_1 and the Camera just as you would usually.
    Start with a simple camera animation, e.g. just a 360 degree
    rotation around the y axis.  Then carefully animate Ball_A's
    rotation so that its front face is always pointing at the camera.
    Next, apply the inverse rotations to Ball_B - when Ball_A has a
    keyframe with (0,90,0) rotation, set a keyframe to Ball_B with
    (0,-90,0).  Finally, set the z-scale of Ball_A to your desired
    "flatness".

Used spheres instead of Balls.


"""
#-----------------------------------------------------------------------------

import poser
import os
import string
scene=poser.Scene()


#-----------------------------------------------------------------------------

# First determine whether the P5-style runtime, with the ball prop
# under Primitives, is present.  This has to come first, because
# otherwise we get a file error one way or the other.
P5 = 0
if os.path.exists("runtime:libraries:props:Primitives:Ball.ppz"):
    P5 = 1

for OneFigure in scene.Figures():
    # See if we've already processed this figure.
    BodyActor=0
    try:
        BodyActor = OneFigure.ParentActor()
    except:
        pass
    try:
        BodyActor = OneFigure.Actor("BODY")
    except:
        pass
    if not BodyActor: continue

    TryParent = BodyActor.Parent()
    if not string.count(TryParent.Name(), "Z-Ball"): # Be sure this figure is not already Balld
        # Create a Ball, name it Z-Ball_B_figname, make it the parent of the figure.
        try:
            if P5:
                scene.LoadLibraryProp("runtime:libraries:props:Primitives:Ball.ppz")
            else:
                scene.LoadLibraryProp("runtime:libraries:props:Prop Types:Ball.pp2")
        except:
            raise "Can't load a ball!"
        BBall = scene.CurrentActor()
        BBall.SetOnOff(0) # Make invisible
        NewString = "Z-Ball_B_%s" % OneFigure.Name()
        BBall.SetName(NewString)
        BodyActor.SetParent(BBall)
        # Create a second Ball, name it Z-Ball_A_figname, make it the parent of the B Ball.
        try:
            if P5:
                scene.LoadLibraryProp("runtime:libraries:props:Primitives:Ball.ppz")
            else:
                scene.LoadLibraryProp("runtime:libraries:props:Prop Types:Ball.pp2")
        except:
            raise "Can't load a ball!"
        ABall = scene.CurrentActor()
        ABall.SetOnOff(0) # Make invisible
        NewString = "Z-Ball_A_%s" % OneFigure.Name()
        ABall.SetName(NewString)
        BBall.SetParent(ABall)

# Now do the same for independent Props.
for OneActor in scene.Actors():
    if not OneActor.IsProp(): continue
    TryParent = OneActor.Parent()
    if not TryParent: continue
    if string.count(OneActor.Name(), "Z-Ball"): continue
    if TryParent.Name() == "UNIVERSE":
        # Using this standard gives us the top-level props.
        # Create a Ball, name it Z-Ball_B_actorname, make it the parent of the figure.
        try:
            scene.LoadLibraryProp("runtime:libraries:props:Prop Types:Ball.pp2")
        except:
            raise "Can't load a ball!"
        BBall = scene.CurrentActor()
        BBall.SetOnOff(0) # Make invisible
        NewString = "Z-Ball_B_%s" % OneActor.Name()
        BBall.SetName(NewString)
        OneActor.SetParent(BBall)
        # Create a Ball, name it Z-Ball_nA, make it the parent of the B Ball.
        try:
            scene.LoadLibraryProp("runtime:libraries:props:Prop Types:Ball.pp2")
        except:
            raise "Can't load a ball!"
        ABall = scene.CurrentActor()
        ABall.SetOnOff(0) # Make invisible
        NewString = "Z-Ball_A_%s" % OneActor.Name()
        ABall.SetName(NewString)
        BBall.SetParent(ABall)

# Now that we've set up everything that needs to be set up, turn the
# Balls by Stewer's recipe.  Assume the currently selected Camera
# will work; I suspect only the Main and Aux will behave right.
# In this step we don't need to locate all the original things;
# just find each of the Z-ball-A and they will lead us to the
# Z-ball-B.

# If the current camera doesn't have a yOrbit or similar,
# just leave the Y alone.
OneCam = scene.CurrentCamera()
UseYorbit = 1
try:
    OrbitYParm = OneCam.ParameterByCode(poser.kParmCodeYROT)
except:
    UseYorbit = 0

# Hold original frame so we can start from 0 and return to orig.
OrigFrame = scene.Frame()
for OneActor in scene.Actors():
    # Find all Z-ball-B by name, then get the A as the
    # parent of the B.
    if string.count(OneActor.Name(), "Z-Ball"):
        try:
            TwoParts = string.split(OneActor.Name(),"_")
        except:
            continue
        TryB = TwoParts[1]
        if TryB == "B":
            BBall = OneActor
            ABall = OneActor.Parent()
            AFacingParm = ABall.Parameter("yRotate")
            AFlatParm = ABall.Parameter("zScale")
            BFacingParm = BBall.Parameter("yRotate")
            for f in range(scene.NumFrames()):
                scene.SetFrame(f)
                # Force the inner Ball (A) to point its front face
                # at the camera.
                if UseYorbit:
                    OrbitY = OrbitYParm.Value()
                else:
                    OrbitY = 0.0
                AFacingParm.SetValue( OrbitY)  # Face the inner Ball to the camera
                BFacingParm.SetValue(-OrbitY)  # Face the outer Ball the opposite way
                AFlatParm.SetValue(0.05)  # Arbitrary value
# Restore orig frame
scene.SetFrame(OrigFrame)
#-----------------------------------------------------------------------------
